import lightgbm as lgb
from attrdict import AttrDict
from catboost import CatBoostClassifier
from sklearn.externals import joblib
from steppy.base import BaseTransformer
from steppy.utils import get_logger
from xgboost import XGBClassifier

from .sklearn_recipes.models import MultilabelEstimators

logger = get_logger()


class CatboostClassifierMultilabel(MultilabelEstimators):
    @property
    def estimator(self):
        return CatBoostClassifier


class XGBoostClassifierMultilabel(MultilabelEstimators):
    @property
    def estimator(self):
        return XGBClassifier


class LightGBM(BaseTransformer):
    def __init__(self, **params):
        self.params = params
        self.training_params = ['number_boosting_rounds', 'early_stopping_rounds']
        self.evaluation_function = None

    @property
    def model_config(self):
        return AttrDict({param: value for param, value in self.params.items()
                         if param not in self.training_params})

    @property
    def training_config(self):
        return AttrDict({param: value for param, value in self.params.items()
                         if param in self.training_params})

    def fit(self,
            X, y,
            X_valid, y_valid,
            feature_names=None,
            categorical_features=None,
            **kwargs):
        train = lgb.Dataset(X,
                            label=y,
                            feature_name=feature_names,
                            categorical_feature=categorical_features)
        valid = lgb.Dataset(X_valid,
                            label=y_valid,
                            feature_name=feature_names,
                            categorical_feature=categorical_features)

        evaluation_results = {}
        self.estimator = lgb.train(self.model_config,
                                   train,
                                   valid_sets=[train, valid],
                                   valid_names=['train', 'valid'],
                                   evals_result=evaluation_results,
                                   num_boost_round=self.training_config.number_boosting_rounds,
                                   early_stopping_rounds=self.training_config.early_stopping_rounds,
                                   verbose_eval=self.model_config.verbose,
                                   feval=self.evaluation_function)
        return self

    def transform(self, X, y=None, **kwargs):
        prediction = self.estimator.predict(X)
        return {'prediction': prediction}

    def load(self, filepath):
        self.estimator = joblib.load(filepath)
        return self

    def save(self, filepath):
        joblib.dump(self.estimator, filepath)



from setuptools import setup

setup(name='steppy-toolkit',
      packages=['toolkit'],
      version='0.1.1',
      description='Set of tools to make your work with steppy faster and more effective.',
      url='https://github.com/minerva-ml/steppy-toolkit',
      download_url='https://github.com/minerva-ml/steppy-toolkit/archive/0.1.1.tar.gz',
      author='Kamil A. Kaczmarek',
      author_email='kamil.kaczmarek@neptune.ml',
      keywords=['machine-learning', 'reproducibility', 'pipeline'],
      license='MIT',
      install_requires=[
          'steppy>=0.1.1',
          'neptune-cli>=2.8.4',
          'tensorflow>=1.8.0',
          'keras>=2.1.6',
          'torch>=0.4.0',
          'torchvision>=0.2.1',
          'attrdict>=2.0.0',
          'catboost>=0.8.1.1',
          'gensim>=3.4.0',
          'ipython>=6.4.0',
          'lightgbm>=2.1.1',
          'nltk>=3.3',
          'numpy>=1.14.3',
          'pandas>=0.23.0',
          'Pillow>=5.1.0',
          'pydot_ng>=1.0.0',
          'pytest>=3.6.0',
          'scikit_learn>=0.19.1',
          'setuptools>=39.2.0',
          'steppy>=0.1.1',
          'xgboost>=0.72'],
      zip_safe=False,
      classifiers=[])
